<?php /**
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */
namespace PrismaLink\Plinkpg\Controller\payment;
use Magento\Framework\App\CsrfAwareActionInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\App\Request\InvalidRequestException;

class ConfirmPaymentStatus extends \Magento\Framework\App\Action\Action implements CsrfAwareActionInterface
{
    protected $resultJsonFactory;
    protected $_checkoutSession;
    protected $connection;
    private $_storeManager;
    private $orderRepository;
    private $table_name="plink_pg";
    private $_logger;
    private $_invoiceService;
    private $_transaction;
    private $_invoiceSender;

    public function __construct(
       \Magento\Framework\App\Action\Context $context,
       \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory,
       \Magento\Checkout\Model\Session $checkoutSession,
       \Magento\Framework\App\ResourceConnection $resourceConnection,
       \Magento\Sales\Api\OrderRepositoryInterface $orderRepository,
       \Magento\Store\Model\StoreManagerInterface $storeManager,
       \PrismaLink\Plinkpg\Helper\PlinkLogHelper $logger,
       \Magento\Sales\Model\Service\InvoiceService $invoiceService,
       \Magento\Framework\DB\Transaction  $transcation,
       \Magento\Sales\Model\Order\Email\Sender\InvoiceSender $invoiceSender
       )
       {    
            $this->_invoiceSender=$invoiceSender;
            $this->_invoiceService=$invoiceService;
            $this->_transaction=$transcation;
            $this->_logger = $logger;
            $this->_storeManager = $storeManager;
            $this->orderRepository = $orderRepository;  
            $this->_checkoutSession = $checkoutSession;
            $this->resultJsonFactory = $resultJsonFactory;
            $this->connection = $resourceConnection->getConnection();

            parent::__construct($context);
       }
    public function execute(){
        
        $result = $this->resultJsonFactory->create();
        $incomingdata=json_decode(file_get_contents('php://input')); //body request
        $this->_logger->logDebug("Incoming data from  ".$this->getClientIp()." (".$_SERVER['REQUEST_METHOD']."): ",$incomingdata);
        if($_SERVER['REQUEST_METHOD'] !== 'POST'){ 
            $result->setHttpResponseCode(\Magento\Framework\Webapi\Exception::HTTP_NOT_FOUND);
            return $result; 
        }else if(!isset($incomingdata)){ 
            $result->setHttpResponseCode(\Magento\Framework\Webapi\Exception::HTTP_FORBIDDEN);
            $result->setData(['error_message' => __('No payload received')]);
            return $result;
        }else{
           try{
              $this->validatePayload($incomingdata);
              if(!$this->checkIsExistOrder($incomingdata->merchant_ref_no)){
                    throw new \Exception("Order id doesn't exist");
                }else{
                    $ref_no=$incomingdata->merchant_ref_no;

                    $order = $this->orderRepository->get($ref_no);
                    if ($order->canInvoice()) {
                        $invoice = $this->_invoiceService->prepareInvoice($order);
                        $invoice->register();
                        $invoice->save();
                        $transactionSave = $this->_transaction->addObject(
                            $invoice
                        )->addObject(
                            $invoice->getOrder()
                        );
                        $transactionSave->save();
                        $this->invoiceSender->send($invoice);
                        //Send Invoice mail to customer
                        $order->addStatusHistoryComment(
                            __('Invoice created for order .'.$order->getIncrementId(), $invoice->getId())
                        )
                            ->setIsCustomerNotified(true)
                            ->save();
                    }

                    $this->updateTransaction($incomingdata);
                    if($incomingdata->payment_status === "SETLD"){
                        echo "Settle";
                        $this->setOrderStatus($ref_no,'complete');
                    }
                    else if($incomingdata->payment_status === "PENDG"){ 
                        echo "Pending";
                        $this->setOrderStatus($ref_no,'pending_payment'); 
                    }
                    else{
                        echo "Cancel";
                        $this->setOrderStatus($ref_no,'canceled'); 
                    }

                    $result->setData([
                        'order_id'=>$incomingdata->merchant_ref_no,
                        'status' => 200,
                        'msg' => __('Update payment status for merchant ID: '.$incomingdata->merchant_ref_no.' Success!')
                    ]);
                    return $result;

                }
           }catch(\Exception $e){
                $this->_logger->logError($e->getMessage());
                $result->setData([
                    'status' => 400,
                    'msg' => $e->getMessage()
                ]);
                return $result;
           }
        }
    }
   
    public function createCsrfValidationException(RequestInterface $request): ? InvalidRequestException
    {
        return null;
    }

    public function validateForCsrf(RequestInterface $request): ?bool
    {
        return true;
    }
     public function getClientIp(){
        if(!empty($_SERVER['HTTP_CLIENT_IP'])){
            //ip from share internet
            $ip = $_SERVER['HTTP_CLIENT_IP'];
        }elseif(!empty($_SERVER['HTTP_X_FORWARDED_FOR'])){
            //ip pass from proxy
            $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
        }else{
            $ip = $_SERVER['REMOTE_ADDR'];
        }
        return $ip;
    }
    public function updateTransaction($incomingdata){
      try{
        $sql = "UPDATE ".$this->table_name." SET ";
        $update_column=[ 
            "paymentStatus"=>$incomingdata->payment_status,
            "plinkRefNo"=>$incomingdata->plink_ref_no,
            "paymentDate"=>$incomingdata->payment_date
        ];
        $last_index = count($update_column) - 1;
        $index = 0;
        foreach($update_column as $column => $value){
            $sql .= $column."='";   
            $index === $last_index ? $sql .= $value."'" : $sql .= $value."'," ; 
            $index++;
        }
        $sql .=" WHERE merchantRefNo='".$incomingdata->merchant_ref_no."';";
        if($this->connection->query($sql)){
            return true;
        }
        return false;
      }catch(\Exception $e){
        throw new \Exception("Something error while updating payment status: ".$e->getMessage());
      }
    }
    public function validatePayload($incomingdata){
        $paymentStatus=['SETLD','PENDG','REJEC'];
        if(!isset($incomingdata->payment_status)||!($incomingdata->plink_ref_no)||!($incomingdata->payment_date)){
            throw new \Exception("Some of the payload is empty");
        }else if(!in_array($incomingdata->payment_status,$paymentStatus)){
            throw new \Exception("Invalid payment status");
        }
    }
    public function checkIsExistOrder($ref_no){
        $sql_check= "SELECT * FROM ".$this->table_name." WHERE merchantRefNo='".$ref_no."' LIMIT 1;";
        $res = $this->connection->fetchAll($sql_check);
        $count=count($res);
        if($count>0){ return true; }
        return false;
    }
    public function getConfig($config_path){
        return $this->scopeConfig->getValue(
            $config_path,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
    }
    public function getBaseUrl(){
        $baseUrl =  $this->_storeManager->getStore()->getBaseUrl();

        return $baseUrl;
    }
    public function setOrderStatus($orderID, $statusCode){
        try{
            $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
            $order = $objectManager->create('\Magento\Sales\Model\Order')
                                    ->load($orderID); 
            $order->setState($statusCode)->setStatus($statusCode);
            $order->save();
            return true;
        } catch (\Exception $e){
            return false;
        }
       
    }
    public function getPlinkMerchantSettings($setting){
        return $this->_objectManager->get('PrismaLink\Plinkpg\Helper\Data')->getConfig('payment/plink_paymentgateway/'.$setting);
    }
}