<?php /**
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */
namespace PrismaLink\Plinkpg\Controller\payment;
use Magento\Sales\Model\Order;


class DoPayment extends \Magento\Framework\App\Action\Action
{
    protected $resultJsonFactory;
    private $_checkoutSession;
    private $_orderRepository;
    private $_logger;
    private $httpHeader;
    private $remoteIp;
    private $_dataHelper;
    private $_infoHelper;
    private $_storeManager;
    private $orderRepository;
    private $session;
    private $connection;
    private $customerSession;
    private $table_name="plink_pg";


    public function __construct(
       \Magento\Framework\App\Action\Context $context,
       \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory,
       \Magento\Checkout\Model\Session $checkoutSession,
       \Magento\Sales\Api\OrderRepositoryInterface $orderRepository,
       \PrismaLink\Plinkpg\Helper\PlinkLogHelper $logger,
       \Magento\Framework\HTTP\Header $httpHeader,
       \Magento\Framework\HTTP\PhpEnvironment\RemoteAddress $remoteIp,
        \PrismaLink\Plinkpg\Helper\PlinkDataHelper $dataHelper,
        \PrismaLink\Plinkpg\Helper\PlinkInfoHelper $infoHelper,
        \Magento\Customer\Model\Session $session,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Framework\App\ResourceConnection $resourceConnection,
        \Magento\Customer\Model\Session $customerSession)
       {
            $this->customerSession =  $customerSession;
            $this->connection = $resourceConnection->getConnection();
            $this->_storeManager = $storeManager;
            $this->_session = $session;    
            $this->_infoHelper = $infoHelper;
            $this->_dataHelper = $dataHelper;
            $this->httpHeader = $httpHeader;
            $this->remoteIp = $remoteIp;
            $this->_logger = $logger; 
            $this->_orderRepository = $orderRepository;
            $this->_checkoutSession = $checkoutSession;
            $this->resultJsonFactory = $resultJsonFactory;
            $this->_session = $session;
            parent::__construct($context);
       }

    
    public function execute(){
        $order=$this->_checkoutSession->getLastRealOrder();
        $order_id=$order->getEntityId();
        $page_id='plinkpaymentgateway';
        
        if(!$order_id){
            $this->_infoHelper->generateMessage("Sorry, your order expired.",$this->getBaseUrl().'checkout/cart/',$this->getBaseUrl(),"Go to checkout page"); 
            $this->_logger->logError("Order ID Specified expired");
        }
        
        $order_details=$this->_orderRepository->get($order_id);


        $actionurl = array(
            "staging"     => 'http://dev.plink.co.id:8280',
            "production"  => 'https://secure2.plink.co.id',
        );

        $frontend_callback_url = array(
            "staging" => "https://www.commerce.plink.co.id",
            "production"  => $this->getBaseUrl(),
        );
        $backend_callback_url = array(
            "staging"     => "https://www.commerce.plink.co.id/plinkpg-paymentflag/",
            "production"  => $this->getBaseUrl().$page_id.'/confirmpaymentstatus',
        );
        
        

        /* Payload */
        $product_details=array();
        $order_items = $order_details->getAllItems();

        //remapping items data
        foreach($order_items as $item_id => $item){
            $product_details[]=array(
                "item_code" => $item->getItemId(), 
                "item_title" => intval($item->get_name()), 
                "quantity" => intval(ceil($item->getQtyOrdered())), 
                "total" => $item->getPrice(), 
                "currency" => "IDR" 
            );
        }

        $other_bills=array(
            array( "title" => 'Tax',"value" => intval(ceil($order_details->getTaxAmount()))),
            array( "title" => 'Shipping',"value" => intval(ceil($order_details->getShippingAmount())))
        );
        
        $shipping_data = $order_details->getShippingAddress();
        $shipping_street='';
        foreach ($shipping_data->getStreet() as $street) {
            if ($shipping_data->getStreet()) {
                $shipping_street .= $street.', ';
            }
        }
        $compose_address =  $shipping_data->getCity().','.
                            $shipping_street.','.
                            $shipping_data->getPostcode().','.
                            $shipping_data->getTelephone().','.
                            $shipping_data->getRegionCode();


        $shipping_address=[
            "address" => $compose_address, 
            "telephoneNumber" => $this->_dataHelper->formatPhoneNumber($shipping_data->getShippingTelephone()), 
            "handphoneNumber" => "" 
        ];
        
        $env=$this->getPlinkMerchantSettings('environment');
        $order_comment = '';
        foreach ($order_details->getStatusHistoryCollection() as $status) {
            if ($status->getComment()) {
                $order_comment .= $status->getComment().' | ';
            }
        }
       
        $body=[
            "invoice_number"=> $order->getIncrementId(),
            "merchant_id" => $this->getPlinkMerchantSettings('merchant_id'), 
            "merchant_key_id" => $this->getPlinkMerchantSettings('company_key_id'), 
            "merchant_ref_no" => $order_id, 
            "transaction_currency" => "IDR", 
            "transaction_amount" => intval(ceil($order->getGrandTotal())), 
            "product_details" => json_encode($product_details), 
            "user_name" => $order_details->getCustomerFirstName().' '.$order_details->getCustomerLastName(), 
            "user_email" =>  $order_details->getCustomerEmail(), 
            "user_phone_number" =>  $this->_dataHelper->formatPhoneNumber($order_details->getShippingAddress()->getTelephone()), 
            "user_id" => $this->customerSession->getCustomer()->getId(), 
            "backend_callback_url" =>$backend_callback_url[$env], 
            "frontend_callback_url" => $frontend_callback_url[$env], 
            "transaction_date_time" => date('Y-m-d H:i:s P'), 
            "transmission_date_time" => date('Y-m-d H:i:s P'), 
            "remarks" => $order_comment,
            "user_device_id" => $this->httpHeader->getHttpUserAgent(), 
            "user_ip_address" => $this->remoteIp->getRemoteAddress() ?? '127.0.0.1', 
            "shipping_details" => json_encode($shipping_address), 
            "payment_method" => "", 
            "other_bills" => json_encode($other_bills)
        ];

        $body_string=json_encode($body,JSON_UNESCAPED_SLASHES|JSON_PRETTY_PRINT);

        $this->insertTransaction(array(
            "ref_no"=>$order_id,
            "user_id"=> $this->customerSession->getCustomer()->getId()&&$order_id,
            "email"=>$order_details->getCustomerEmail()
        ));

         /*Create HMAC and update header */
        $secret_key = $this->getPlinkMerchantSettings('secret_key'); //secret key
        $payload = json_encode($body,JSON_UNESCAPED_SLASHES|JSON_PRETTY_PRINT);
        $token=hash_hmac('sha256', $payload, $secret_key);

        $header=array(
            'Content-Type: application/json',
            'Accept: application/json',
            'mac:'.$token  
        );

        $data=array(
            'order_id'=>$order_id,
            'base_url' => $actionurl[$env],
            'end_point' => '/integration/transaction/api/submit-trx',
            'body' => $body_string,
            'header' => $header
        );
        $checkout_url=$this->getBaseUrl().'checkout/cart/';
        try{
            $isPaid=$this->inquryPaymentStatus($order_id);

            if($isPaid === null){
                $this->_logger->logDebug("Call to PG backend:",$data);
                $response=$this->_dataHelper->callToPg($data);
                $resData=json_decode($response);
                $this->_logger->logDebug("PG response success:",$resData);
                if($resData->response_code === "MBDD00"){
                    $iframe_url=$data['base_url'].$resData->payment_page_url;
                    $this->_infoHelper->generatePaymentPage($iframe_url,$this->getBaseUrl());
                }else{
                    $this->_infoHelper->generateMessage("Something went wrong:\n".$resData->response_description.".",$this->getBaseUrl(),$checkout_url);
                }
            }else{ 
                $messageData=array(
                    'SETLD'=>[
                                'msg' => 'Payment succeed. Thank u.',
                                'type' => 'success',
                                'redirect_url' => $this->getBaseUrl().'checkout/cart/',
                                'redirect_msg' => 'Go to checkout page'
                            ],
                    'PENDG'=>[
                                'msg' => 'Sorry your payment is still pending, please check your email or payment history to track your order. Thank u.',
                                'type' => 'warning',
                                'redirect_url' => $this->getBaseUrl().'checkout/cart/',
                                'redirect_msg' => 'Go to checkout page'
                             ],
                    'REJEC'=>[
                                'msg' => 'Sorry your payment is failed',
                                'type' => 'error',
                                'redirect_url' => $this->getBaseUrl(),
                                'redirect_msg' => 'Continue to homepage'
                             ],
                );

                $message=$messageData[$isPaid];
                $msg=$message['msg'];
                $redirectMessage=$message['redirect_msg'];
                $redirectUrl=$message['redirect_url'];
                $baseUrl=$this->getBaseUrl();
                $msgType=$message['type'];
                $buttonMsg="Go to checkout page";
                $this->setOrderStatus($order_id,"pending");
                $this->_infoHelper->generateMessage($msg,$redirectUrl,$baseUrl,$buttonMsg,$msgType);                
            }
        }catch(\Exception $exeption){
            $this->_infoHelper->generateMessage('Something went wrong.  Please try again later',$this->getBaseUrl(),$checkout_url);
        }
    } 
    public function inquryPaymentStatus($ref_no){
        $sql_check= "SELECT * FROM ".$this->table_name." WHERE merchantRefNo='".$ref_no."' LIMIT 1;";
        $res = $this->connection->fetchAll($sql_check);
        $count=count($res);
        if($count>0){
            $row = $res[0];
            $paymentStatus=$row['paymentStatus'];
            if(is_null($paymentStatus) || empty($paymentStatus)){
                return null;
            }else{
                return $paymentStatus;
            }

        }
        throw new ErrorException("No order data exists");
    }
    public function insertTransaction($fields){
        $sql_insert = "INSERT INTO ".$this->table_name." (merchantRefNo,plinkRefNo,user_id,amount,email,transactionDate,paymentStatus,description) 
        VALUES('".$fields['ref_no']."','','".$fields['user_id']."','','".$fields['email']."','".date('Y-m-d H:i:s')."','','');";
        $sql_check= "SELECT * FROM ".$this->table_name." WHERE merchantRefNo='".$fields['ref_no']."' LIMIT 1;";
        //check if data exists
        $res = $this->connection->fetchAll($sql_check);
        $total_rows = count($res);
        if($total_rows<1){
            if($this->connection->query($sql_insert)){
                return true;
            }
            return false;
        }
        return false;
    }
    public function getConfig($config_path){
        return $this->scopeConfig->getValue(
            $config_path,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
    }
    public function getBaseUrl(){
        $baseUrl =  $this->_storeManager->getStore()->getBaseUrl();

        return $baseUrl;
    } 
    public function checkIsLoggedIn(){
        if ($this->_session->isLoggedIn()) {
            return true; 
        } 
        return false;
    }
    public function setOrderStatus($orderID, $statusCode){
        try{
            // obtain the order with the order ID
            $order = $this->_orderRepository->get($orderID);
            $order->setState($statusCode)->setStatus($statusCode);
            $this->_orderRepository->save($order);
            return true;
        } catch (\Exception $e){
            // add some logging here
            return false;
        }
    }
    public function getPlinkMerchantSettings($setting){
        return $this->_objectManager->get('PrismaLink\Plinkpg\Helper\Data')->getConfig('payment/plink_paymentgateway/'.$setting);
    }
}